                                   /*----------------------------------------------------------------------------
 * Name:    main.c
 * Purpose: LED driver
 ----------------------------------------------------------------------------*/

//------------------------------ Header files ----------------------------------
#include <stdio.h>
#include <LPC407x_8x_177x_8x.h>
#include "delay.h"
#include "UART.h"
#include "GPIO.h"
#include "PWM.h"
#include "SPI.h"
#include "TIMER.h"
#include <RTL.h>


//------------------------------ Define constants ----------------------------------
#define BUFFERSPISIZE		16
#define BUFFERFCESIZE		8


//-------------------------- RTOS task definitions -----------------------------
	OS_TID t_blink;
	OS_TID t_uart_tx;
	OS_TID t_interpreter;
	OS_TID t_spi247;
	OS_TID t_spi787;
	OS_TID t_dc247;
	OS_TID t_wb1;
	OS_TID t_wb2;
	OS_TID t_wb3;
	OS_TID t_wb4;
	OS_TID t_dz1;
	OS_TID t_dz2;
	OS_TID t_dz3;
	OS_TID t_dz4;
	OS_TID t_bl1;
	OS_TID t_bl2;
	OS_TID t_bl3;
	OS_TID t_bl4;


//------------------------ RTOS semaphores definitions --------------------------
	OS_SEM Semspi247;
	OS_SEM Semspi787;
	OS_SEM Semdc247;

	
//------------------------ RTOS mutexes definitions --------------------------
	OS_MUT Mutspi247;
	OS_MUT Mutspi787;
	OS_MUT Mutuart_tx;


//------------------------------- UART Buffers ----------------------------------
	uint8_t UART_RXBuffer[BUFFERSIZE];								
	uint8_t UART_TXBuffer[BUFFERSIZE];
	uint8_t UART_RXReadBuff = 0;
	uint8_t UART_RXWriteBuff = 0;
	uint8_t UART_TXReadBuff = 0;
	uint8_t UART_TXWriteBuff = 0;
	uint8_t UART_RXDataCount = 0;
	uint8_t UART_TXDataCount = 0;
	uint8_t BufferDataSize = 0;
	uint8_t UART_RcvData = 0;
	
	
//-------------------------------- SPI Buffers -----------------------------------	
	uint8_t SPI787_Buffer [BUFFERSPISIZE][4];
	uint8_t SPI787_WriteBuff = 0;
	uint8_t SPI787_ReadBuff = 0;	
	uint8_t SPI247_Buffer [BUFFERSPISIZE][5];	
	uint8_t SPI247_WriteBuff = 1;
	uint8_t SPI247_ReadBuff = 0;	
	uint8_t DC247_Buffer [17];	
	uint32_t SPI_RcvData = 0;
	uint32_t SPI_RcvDataLow = 0;
	uint32_t SPIDataDel = 0;
	
	
//------------------------------ Source variables ---------------------------------
	uint8_t targetIC = 0;
	uint8_t R_W = 0;
	uint8_t BUCKx_ISENS_RNG = 0x0;
	uint8_t BUCKx_ISENS_D3 = 0x0;
	uint8_t BUCKx_ISENS_D2 = 0x0;
	uint8_t BUCKx_ISENS_D1 = 0x0;
	
	
//----------------------------- Dimming variables ---------------------------------
	uint16_t DCs[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	uint8_t TR[12] = {15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15};
	uint8_t TRoff[12] = {15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15};
	uint8_t lastLevel3[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	uint8_t lastLevel4[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};	
	uint8_t lastLevel[8][12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, };
	uint16_t ON[12] = {158, 230, 302, 374, 446, 518, 590, 662, 734, 806, 878, 950};
	uint16_t OFF[12] = {208, 280, 352, 424, 496, 568, 640, 712, 784, 856, 928, 1000};
	uint8_t LEDcount = 12;	

	
//---------------- Wiping blinker settings and variables --------------------------	
	uint8_t wb_start[8] = {0, 0, 0, 0, 0, 0, 0, 0}; 
	uint8_t wb_stop[8] = {0, 0, 0, 0, 0, 0, 0, 0}; 
	uint8_t wb_per[8] = {0, 0, 0, 0, 0, 0, 0, 0}; 
	uint8_t wb_counter[8] = {0, 0, 0, 0, 0, 0, 0, 0}; 
	
	
//------------------ Dark zone settings and variables -----------------------------		
	uint8_t dz_per[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	uint8_t dz_gap[8] = {0, 0, 0, 0, 0, 0, 0, 0};	
	uint8_t dz_counter[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	
	
//----------------- Bending light settings and variables --------------------------		
	uint8_t bl_per[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	uint8_t bl_gap[8] = {0, 0, 0, 0, 0, 0, 0, 0};	
	uint8_t bl_counter[8] = {0, 0, 0, 0, 0, 0, 0, 0};	
	
	
//-------------------------- Auxiliary variables ----------------------------------	
	uint8_t newtarget = 255;
	uint32_t temp;
	uint8_t i, j;
	
	
//------------------------ Prototypes of functions ---------------------------------	
void dimming (uint16_t *DCdimm, uint8_t LEDcountdimm, uint16_t *ONdimm, uint16_t *OFFdimm, uint8_t *TRdimm, uint8_t *lastStdimm);	
	
	
//-------------------------- TASK: blink -- (Begin) ---------------------------------
// Blinking with LED D5	
	__task void blink (void)
{
		while(1)
		{
			LED_off(5);												// turn off LED5
			os_dly_wait (500);                // wait for 500 ms
			LED_on(5);												// turn on LED5
			os_dly_wait (500);                // wait for 500 ms
		}
}
//-------------------------- TASK: blink -- (End) ----------------------------------


//-------------------------- TASK: wb -- (Begin) ----------------------------------
// LED effect - Wiping blinker	
__task void wb (void)
{
		uint8_t target = newtarget;																					// target device
		float wb_dt = 0; 																										// time difference
	
		while(1)
		{
				wb_dt = (float)(wb_stop[target] - wb_start[target])/12.0;	
			
				if (wb_counter[target] >= wb_per[target])								  			// counter overflow
					wb_counter[target] = 0;
				else
					wb_counter[target]++;																					// increment counter
				
				
				if (wb_counter[target] < wb_start[target])											// all LEDs OFF
				{
					for (i=0; i<12; i++)
					{
							DCs[i] = 1023;
					}
					j=0;
				}
				else if (wb_counter[target] < wb_stop[target])									//all LEDs dimmed
				{	
						for (i=0; i<12; i++)
						{
								if (wb_counter[target] < (wb_start[target] + i*wb_dt))
										DCs[i] = 1023;
								else if (wb_counter[target] < (wb_start[target] + (i+1)*wb_dt))
										DCs[i] = (wb_start[target] + (i+1)*wb_dt - wb_counter[target])/wb_dt*1023;
								else 
									DCs[i] = 0;
						}			
				}
				else																														// all LEDs ON
				{
					for (i=0; i<12; i++)
					{
							DCs[i] = 0;
					}
				}
					
				dimming(DCs, LEDcount, ON, OFF, TR, lastLevel[target]);					// ON, OFF, TR values computation using Dimming algorithm
					
				os_mut_wait(&Mutspi247, 0xFFFF);																// wait for mutex
				
				SPI_SendFrame(target + 3, 1, 0xCF, 0, 0, 0);										// send SPI frame to pixel light
					
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data	
									
				SPI_SendFrame(target + 3, 1, 0xCC, ((SPI_RcvData)&(0xFB)), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);	
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data						
									
				for(i=0; i<12; i++)																							// send ON, OFF and TR values to pixel light
				{
						SPI_SendFrame(target + 3, 1, i + 0xC0, ((OFF[i]<<4)+TR[i])&0xFF, ((ON[i]<<6)+(OFF[i]>>4))&0xFF, (ON[i]>>2)&0xFF);  
						while ((LPC_SSP2->SR & 0x4)== 0x0){}												// wait until frame is received
						SPI_RcvData = (LPC_SSP2->DR)<<16;														// shift received data
						while ((LPC_SSP2->SR & 0x4)== 0x0){}												// wait until frame is received
						SPI_RcvData += LPC_SSP2->DR;																// sum received data	
				}
									
				SPI_SendFrame(target + 3, 1, 0xCC, ((SPI_RcvData)&0xFF)|(1<<2), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);			//set bit
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data	
				
				os_mut_release(&Mutspi247);																			// release mutex
					
				os_dly_wait(20);
		
		} // end while(1)
}
//--------------------------- TASK: wb -- (End) -----------------------------------


//-------------------------- TASK: dz -- (Begin) ----------------------------------
// LED effect - Dark Zone	
__task void dz (void)
{
		uint8_t target = newtarget;																					// target device
		float edge = 0;																											// first edge of dark zone
		uint8_t dir = 1;																										// direction of move (LED1->LED12 = 1),(LED12->LED1 = 0) 
		
	
		while(1)
		{
				if (dz_counter[target] >= dz_per[target])											  // counter overflow at LED12, change direction
				{
					dir = 0;
					dz_counter[target]--;
				}
				else if (dz_counter[target] <= 0)																// counter overflow at LED1, change direction
				{
					dir = 1;
					dz_counter[target]++;
				}
				else if (dir == 1)																							// increment counter when dir = 1
					dz_counter[target]++;
				else																														// decrement counter when dir = 0
					dz_counter[target]--;
				
				edge = dz_counter[target]/(float)dz_per[target]*(12.0 + dz_gap[target]);							// computate first edge of dark zone
				
				for (i = 0; i < 12; i++)																				// set brightness of LED according to move of dark zone
				{
						if(edge <= i)																																			// LED ON
							DCs[i] = 0;
						else if ((edge > i) && (edge <= (i + 1)))																					// falling slope
							DCs[i] = (edge - i) * 1023;
						else if ((edge > (i+1)) && ((edge - dz_gap[target]) <= i))												// LED OFF
							DCs[i] = 1023;
						else if (((edge - dz_gap[target]) > i) && ((edge - dz_gap[target]) <= (i + 1)))		// rising slope
							DCs[i] = (i+1-(edge - dz_gap[target]))*1023;
						else																																							// LED ON
							DCs[i] = 0;
				}
				
					
				dimming(DCs, LEDcount, ON, OFF, TR, lastLevel[target]);					// ON, OFF, TR values computation using Dimming algorithm
					
				os_mut_wait(&Mutspi247, 0xFFFF);																// wait for mutex
				
				SPI_SendFrame(target + 3, 1, 0xCF, 0, 0, 0);										// send SPI frame to pixel light
					
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data	
									
				SPI_SendFrame(target + 3, 1, 0xCC, ((SPI_RcvData)&(0xFB)), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);	
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data						
									
				for(i=0; i<12; i++)																							// send ON, OFF and TR values to pixel light
				{
						SPI_SendFrame(target + 3, 1, i + 0xC0, ((OFF[i]<<4)+TR[i])&0xFF, ((ON[i]<<6)+(OFF[i]>>4))&0xFF, (ON[i]>>2)&0xFF);  
						while ((LPC_SSP2->SR & 0x4)== 0x0){}												// wait until frame is received
						SPI_RcvData = (LPC_SSP2->DR)<<16;														// shift received data
						while ((LPC_SSP2->SR & 0x4)== 0x0){}												// wait until frame is received
						SPI_RcvData += LPC_SSP2->DR;																// sum received data	
				}
									
				SPI_SendFrame(target + 3, 1, 0xCC, ((SPI_RcvData)&0xFF)|(1<<2), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);			//set bit
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data	
				
				os_mut_release(&Mutspi247);																			// release mutex
					
				os_dly_wait(20);
		
		} // end while(1)
}
//--------------------------- TASK: dz -- (End) -----------------------------------


//-------------------------- TASK: bl -- (Begin) ----------------------------------
// LED effect - Bending Light
__task void bl (void)
{
		uint8_t target = newtarget;																					// target device
		float edge = 0;																											// first edge of dark zone
		uint8_t dir = 1;																										// direction of move (LED1->LED12 = 1),(LED12->LED1 = 0) 
		
	
		while(1)
		{
				if (bl_counter[target] >= bl_per[target])											  // counter overflow at LED12, change direction
				{
					dir = 0;
					bl_counter[target]--;
				}
				else if (bl_counter[target] <= 0)											 					 // counter overflow at LED1, change direction
				{
					dir = 1;
					bl_counter[target]++;
				}
				else if (dir == 1)																							// increment counter when dir = 1
					bl_counter[target]++;
				else																														// decrement counter when dir = 0
					bl_counter[target]--;
				
				edge = bl_counter[target]/(float)bl_per[target]*(12.0 - bl_gap[target]) + bl_gap[target];		// computate first edge of bending light
				
				for (i = 0; i < 12; i++)																				// set brightness of LED according to move of dark zone
				{
						if(edge <= i)																																			// LED OFF
							DCs[i] = 1023;
						else if ((edge > i) && (edge <= (i + 1)))																					// rising slope
							DCs[i] = (1 + i - edge) * 1023;
						else if ((edge > (i+1)) && ((edge - bl_gap[target]) <= i))												// LED ON
							DCs[i] = 0;
						else if (((edge - bl_gap[target]) > i) && ((edge - bl_gap[target]) <= (i + 1)))		// falling slope
							DCs[i] = ((edge - bl_gap[target]) - i)*1023;
						else																																							// LED OFF
							DCs[i] = 1023;
				}
					
				dimming(DCs, LEDcount, ON, OFF, TR, lastLevel[target]);					// ON, OFF, TR values computation using Dimming algorithm
					
				os_mut_wait(&Mutspi247, 0xFFFF);																// wait for mutex
				
				SPI_SendFrame(target + 3, 1, 0xCF, 0, 0, 0);										// send SPI frame to pixel light
					
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data	
									
				SPI_SendFrame(target + 3, 1, 0xCC, ((SPI_RcvData)&(0xFB)), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);	
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data						
									
				for(i=0; i<12; i++)																							// send ON, OFF and TR values to pixel light
				{
						SPI_SendFrame(target + 3, 1, i + 0xC0, ((OFF[i]<<4)+TR[i])&0xFF, ((ON[i]<<6)+(OFF[i]>>4))&0xFF, (ON[i]>>2)&0xFF);  
						while ((LPC_SSP2->SR & 0x4)== 0x0){}												// wait until frame is received
						SPI_RcvData = (LPC_SSP2->DR)<<16;														// shift received data
						while ((LPC_SSP2->SR & 0x4)== 0x0){}												// wait until frame is received
						SPI_RcvData += LPC_SSP2->DR;																// sum received data	
				}
									
				SPI_SendFrame(target + 3, 1, 0xCC, ((SPI_RcvData)&0xFF)|(1<<2), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);			//set bit
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData = (LPC_SSP2->DR)<<16;																// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}														// wait until frame is received
				SPI_RcvData += LPC_SSP2->DR;																		// sum received data	
							
				os_mut_release(&Mutspi247);																			// release mutex
					
				os_dly_wait(20);
		
		} // end while(1)
}
//--------------------------- TASK: bl -- (End) -----------------------------------


//----------------------- TASK: interpreter -- (Begin) ------------------------------
// Define and run functions according data, received via UART
__task void interpreter (void)
{
		while(1)
		{		
				if (UART_RXDataCount > 0)
				{
					UART_RXDataCount--;

					switch(UART_RXBuffer[UART_RXReadBuff])																				// type of operation
					{
							case 1:																																		// SPI transfer
							{

									while(UART_RXDataCount==0) {}																					// wait until data are ready
											UART_RXDataCount--;

											if (UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)] < 3)				// NCV787xx Read
											{
													while(UART_RXDataCount < 1) {}																// wait for 1 data frame
													UART_RXDataCount = UART_RXDataCount - 1;
													SPI787_Buffer[SPI787_WriteBuff][0] = UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)];		//copy data to SPI buffer
													SPI787_Buffer[SPI787_WriteBuff][1] = UART_RXBuffer[(UART_RXReadBuff+2)&(BUFFERSIZE-1)];
													UART_RXReadBuff = (UART_RXReadBuff + 3) & (BUFFERSIZE - 1);
													SPI787_WriteBuff = (SPI787_WriteBuff + 1) & (BUFFERSPISIZE - 1);
													os_sem_send(&Semspi787);																			// run spi787 task via semaphore
											}
											else if (((UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)] & 0x07) < 3) 
												&& (UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)] & 0x08))				// NCV787xx Write
											{
													while(UART_RXDataCount < 3) {}																// wait for 3 data frames	
													UART_RXDataCount = UART_RXDataCount - 3;
													SPI787_Buffer[SPI787_WriteBuff][0] = UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)];		//copy data to SPI buffer
													SPI787_Buffer[SPI787_WriteBuff][1] = UART_RXBuffer[(UART_RXReadBuff+2)&(BUFFERSIZE-1)];
													SPI787_Buffer[SPI787_WriteBuff][2] = UART_RXBuffer[(UART_RXReadBuff+3)&(BUFFERSIZE-1)];
													SPI787_Buffer[SPI787_WriteBuff][3] = UART_RXBuffer[(UART_RXReadBuff+4)&(BUFFERSIZE-1)];														
													UART_RXReadBuff = (UART_RXReadBuff + 5) & (BUFFERSIZE - 1);
													SPI787_WriteBuff = (SPI787_WriteBuff + 1) & (BUFFERSPISIZE - 1);																								
													os_sem_send(&Semspi787);																			// run spi787 task via semaphore												
											}
											else if (((UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)]&0x07) == 3)
												|| ((UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)]&0x07) == 4))			// odstraneno
											{									
													while(UART_RXDataCount < 4) {}																// wait for 4 data frames	
													UART_RXDataCount = UART_RXDataCount - 4;
													SPI247_Buffer[SPI247_WriteBuff][0] = UART_RXBuffer[(UART_RXReadBuff+1)&(BUFFERSIZE-1)];		//copy data to SPI buffer
													SPI247_Buffer[SPI247_WriteBuff][1] = UART_RXBuffer[(UART_RXReadBuff+2)&(BUFFERSIZE-1)];
													SPI247_Buffer[SPI247_WriteBuff][2] = UART_RXBuffer[(UART_RXReadBuff+3)&(BUFFERSIZE-1)];
													SPI247_Buffer[SPI247_WriteBuff][3] = UART_RXBuffer[(UART_RXReadBuff+4)&(BUFFERSIZE-1)];
													SPI247_Buffer[SPI247_WriteBuff][4] = UART_RXBuffer[(UART_RXReadBuff+5)&(BUFFERSIZE-1)];														
													UART_RXReadBuff = (UART_RXReadBuff + 6) & (BUFFERSIZE - 1);
													SPI247_WriteBuff = (SPI247_WriteBuff + 1) & (BUFFERSPISIZE - 1);														
													os_sem_send(&Semspi247);																			// run spi247 task via semaphore		
											}
									break;						
							}
							
							case 2:																																		// change ENABLE signal
							{
									while(UART_RXDataCount < 1) {}																				// wait for 1 data frame
									UART_RXDataCount--;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									if (UART_RXBuffer[UART_RXReadBuff] == 0x01)														// ENABLE -> 1 
											LPC_GPIO1->SET = 0x00004000;																			// ENABLE -> 1
									if (UART_RXBuffer[UART_RXReadBuff] == 0x02)														// ENABLE -> 0 	
											LPC_GPIO1->CLR = 0x00004000;																			// ENABLE -> 0
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									break;						
							}
							
							case 3:																																		// BSTSYNC signal
							{
									while(UART_RXDataCount < 3) {}																				// wait for 3 data frames
									UART_RXDataCount = UART_RXDataCount - 3;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);	
									if (UART_RXBuffer[UART_RXReadBuff] == 0x01)														// BSTSYNC ON						
											LPC_TIM0->TCR = 0x00000001;          															// BSTSYNC ON - allow TC																
									else														 																			// BSTSYNC OFF
									{
											LPC_TIM0->TCR = 0x00000000;          															// stop TC 
											LPC_TIM0->TC = 0x00000000;           															// reset TC	
											LPC_TIM0->PC = 0x00000000;           															// reset PC	
									}					
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);																							
									temp = UART_RXBuffer[(UART_RXReadBuff + 1) & (BUFFERSIZE - 1)];				// BSTSYNC frequency - high byte
									temp = temp << 8;
									temp = temp + UART_RXBuffer[UART_RXReadBuff];													// BSTSYNC frequency - high + low byte										
									UART_RXReadBuff = (UART_RXReadBuff + 2) & (BUFFERSIZE - 1);																			
									LPC_TIM0->MR0 = temp; 																								// frequency -> MR 
									LPC_TIM0->TC = 0x00000000;           																	// reset TC	
									LPC_TIM0->PC = 0x00000000;           																	// reset PC						
									break;						
							}
							
							case 4:																																		// LEDCTRL signal
							{
									while(UART_RXDataCount < 8) {}																				// wait for 8 data frames
									UART_RXDataCount = UART_RXDataCount - 8;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									LEDCTRL(UART_RXBuffer[(UART_RXReadBuff + 0) & (BUFFERSIZE - 1)], UART_RXBuffer[(UART_RXReadBuff + 1) & (BUFFERSIZE - 1)],	// use LEDCTRL function
										UART_RXBuffer[(UART_RXReadBuff + 2) & (BUFFERSIZE - 1)], UART_RXBuffer[(UART_RXReadBuff + 3) & (BUFFERSIZE - 1)],
										UART_RXBuffer[(UART_RXReadBuff + 4) & (BUFFERSIZE - 1)], UART_RXBuffer[(UART_RXReadBuff + 5) & (BUFFERSIZE - 1)],
										UART_RXBuffer[(UART_RXReadBuff + 6) & (BUFFERSIZE - 1)], UART_RXBuffer[(UART_RXReadBuff + 7) & (BUFFERSIZE - 1)]);
									UART_RXReadBuff = (UART_RXReadBuff + 8) & (BUFFERSIZE - 1);	
									break;						
							}
							
							case 5:																																		// FSO/TST2 signal
							{
									while(UART_RXDataCount < 1) {}																				// wait for 1 data frame
									UART_RXDataCount--;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									if (UART_RXBuffer[UART_RXReadBuff] == 0x01)														// FSO/TST2 -> 1 
									{
											LPC_GPIO1->SET = 0x00000400;																			// FSO/TST2 -> 1
											LPC_GPIO1->CLR = 0x00040000;								
									}
									if (UART_RXBuffer[UART_RXReadBuff] == 0x02)														// FSO/TST2 -> 0 	
									{
											LPC_GPIO1->CLR = 0x00000400;																			// FSO/TST2 -> 0
											LPC_GPIO1->SET = 0x00040000;
									}	
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									break;						
							}

							case 6:																																		// RSTB signal
							{
									while(UART_RXDataCount < 1) {}																				// wait for 1 data frame
									UART_RXDataCount--;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									if (UART_RXBuffer[UART_RXReadBuff] == 0x01)														// RSTB -> 1 
											LPC_GPIO1->SET = 0x00008000;																			// RSTB -> 1										
									if (UART_RXBuffer[UART_RXReadBuff] == 0x02)														// RSTB -> 0 	
											LPC_GPIO1->CLR = 0x00008000;																			// RSTB -> 0
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									break;							
							}
							
							case 7:																																		// polling message
							{

									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);	
									
									os_mut_wait(&Mutuart_tx, 0xFFFF);																			// wait for mutex
									UART_SendFrame (0x07);																								// send reply, frame 0x07
									UART_SendFrame (0xAB);																								// send version of firmware = 0xAB
									os_mut_release(&Mutuart_tx);																					// release mutex
									break;
							}
							
							case 8:																																		// control LEDs with duty-cycles of their brightness
							{
									while(UART_RXDataCount < 16) {}																				// wait for 16 data frames
									UART_RXDataCount = UART_RXDataCount - 16;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									
									for (i=0; i<16; i++)																									// copy frames into DC247 buffer
										DC247_Buffer[i] = UART_RXBuffer[(UART_RXReadBuff + i) & (BUFFERSIZE - 1)];
										
									UART_RXReadBuff = (UART_RXReadBuff + 16) & (BUFFERSIZE - 1);	
									os_sem_send(&Semdc247);																								// run dc247 task via semaphore
									break;						
							}					
							
							case 9:																																		// stand-alone function - Wiping blinker
							{
								
									while(UART_RXDataCount < 4) {}																				// wait for 4 data frames
									UART_RXDataCount = UART_RXDataCount - 4;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									
									newtarget = (UART_RXBuffer[UART_RXReadBuff] >> 2) & 0x0F;
									wb_start[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 1) & (BUFFERSIZE - 1)];		// settings of function
									wb_stop[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 2) & (BUFFERSIZE - 1)];
									wb_per[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 3) & (BUFFERSIZE - 1)];		
										
									if ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0xC0) == 0)						// create task according to target device
									{
											switch ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0x0F))					// create task according to target device
											{
												case 3:
														t_wb1 = os_tsk_create(wb, 0);
														break;
												case 4:
														t_wb2 = os_tsk_create(wb, 0);
														break;
												case 5:
														t_wb3 = os_tsk_create(wb, 0);
														break;
												case 6:
														t_wb4 = os_tsk_create(wb, 0);
														break;												
												default:
														break;
											}		
									}
									if ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0xC0) == 0x80)					// delete task according to target device
									{	
											switch ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0x0F))					// delete task according to target device
											{
												case 3:
														t_wb1 = os_tsk_delete(t_wb1);
														break;
												case 4:
														t_wb2 = os_tsk_delete(t_wb2);
														break;
												case 5:
														t_wb3 = os_tsk_delete(t_wb3);
														break;
												case 6:
														t_wb4 = os_tsk_delete(t_wb4);
														break;												
												default:
														break;
											}													
									}		
										
									UART_RXReadBuff = (UART_RXReadBuff + 4) & (BUFFERSIZE - 1);	
									break;						
							}
							
							case 10:																																		// stand-alone function - Dark Zone
							{
								
									while(UART_RXDataCount < 3) {}																					// wait for 3 data frames
									UART_RXDataCount = UART_RXDataCount - 3;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									
									newtarget = (UART_RXBuffer[UART_RXReadBuff] >> 2) & 0x0F;
									dz_gap[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 1) & (BUFFERSIZE - 1)];		// settings of function
									dz_per[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 2) & (BUFFERSIZE - 1)];
	
										
									if ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0xC0) == 0)					// create task according to target device
									{
											switch ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0x0F))				// create task according to target device
											{
												case 3:
														t_dz1 = os_tsk_create(dz, 0);
														break;
												case 4:
														t_dz2 = os_tsk_create(dz, 0);
														break;
												case 5:
														t_dz3 = os_tsk_create(dz, 0);
														break;
												case 6:
														t_dz4 = os_tsk_create(dz, 0);
														break;												
												default:
														break;
											}		
									}
									if ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0xC0) == 0x80)				// delete task according to target device
									{	
											switch ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0x0F))				// delete task according to target device
											{
												case 3:
														t_dz1 = os_tsk_delete(t_dz1);
														break;
												case 4:
														t_dz2 = os_tsk_delete(t_dz2);
														break;
												case 5:
														t_dz3 = os_tsk_delete(t_dz3);
														break;
												case 6:
														t_dz4 = os_tsk_delete(t_dz4);
														break;												
												default:
														break;
											}													
									}		
										
									UART_RXReadBuff = (UART_RXReadBuff + 3) & (BUFFERSIZE - 1);	
									break;						
							}	
							
							case 11:																																		// stand-alone function - Bending Light
							{
								
									while(UART_RXDataCount < 3) {}																					// wait for 3 data frames
									UART_RXDataCount = UART_RXDataCount - 3;
									UART_RXReadBuff = (UART_RXReadBuff + 1) & (BUFFERSIZE - 1);
									
									newtarget = (UART_RXBuffer[UART_RXReadBuff] >> 2) & 0x0F;
									bl_gap[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 1) & (BUFFERSIZE - 1)];		// settings of function
									bl_per[newtarget] = UART_RXBuffer[(UART_RXReadBuff + 2) & (BUFFERSIZE - 1)];
	
										
									if ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0xC0) == 0)					// create task according to target device
									{
											switch ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0x0F))				// create task according to target device
											{
												case 3:
														t_bl1 = os_tsk_create(bl, 0);
														break;
												case 4:
														t_bl2 = os_tsk_create(bl, 0);
														break;
												case 5:
														t_bl3 = os_tsk_create(bl, 0);
														break;
												case 6:
														t_bl4 = os_tsk_create(bl, 0);
														break;												
												default:
														break;
											}		
									}
									if ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0xC0) == 0x80)				// delete task according to target device
									{	
											switch ((UART_RXBuffer[(UART_RXReadBuff) & (BUFFERSIZE - 1)] & 0x0F))				// delete task according to target device
											{
												case 3:
														t_bl1 = os_tsk_delete(t_bl1);
														break;
												case 4:
														t_bl2 = os_tsk_delete(t_bl2);
														break;
												case 5:
														t_bl3 = os_tsk_delete(t_bl3);
														break;
												case 6:
														t_bl4 = os_tsk_delete(t_bl4);
														break;												
												default:
														break;
											}													
									}		
										
									UART_RXReadBuff = (UART_RXReadBuff + 3) & (BUFFERSIZE - 1);	
									break;						
							}							
							
							default:
							{}
						}		// end of switch
					}		// end of if (UART_RXDataCount > 0)
				
					else																																										// no receive data
					{
					os_dly_wait(10);
					}
		}		// end of while(1)

}		// end of task
//----------------------- TASK: interpreter -- (End) -------------------------------


//------------------------- TASK: uart_tx -- (Begin) -------------------------------
// Send data via UART using 16 bytes TX buffer
__task void uart_tx (void)
{
		uint8_t limit;																													// max size of TX buffer
		while(1)
		{
				limit = 16;
				while(UART_TXDataCount > 0)																					// run until data are sent
				{

						LPC_UART0->THR = UART_TXBuffer[UART_TXReadBuff];								// copy one frame to buffer
						UART_TXReadBuff = (UART_TXReadBuff + 1) & (BUFFERSIZE-1);
						UART_TXDataCount--;
						if (limit-- == 0) break;																				// break if buffer limit overflow
				}
				os_dly_wait (10);
		}
}
//------------------------- TASK: uart_tx -- (End) ---------------------------------


//------------------------- TASK: spi247 -- (Begin) --------------------------------
// SPI communication with pixel light
__task void spi247 (void)
{   
		while(1)
		{
				os_sem_wait(&Semspi247, 0xFFFF);																						// wait for data (driven by semaphore)
				SPI247_ReadBuff = (SPI247_ReadBuff+1) & (BUFFERSPISIZE - 1);
				os_mut_wait(&Mutspi247, 0xFFFF);																						// wait for mutex
				SPI_SendFrame((SPI247_Buffer[SPI247_ReadBuff][0] & 0x07), 0, SPI247_Buffer[SPI247_ReadBuff][1], SPI247_Buffer[SPI247_ReadBuff][2],
					SPI247_Buffer[SPI247_ReadBuff][3], SPI247_Buffer[SPI247_ReadBuff][4]); 		// send SPI frame 
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until first SPI frame is received 								
				SPI_RcvData = LPC_SSP2->DR;																									// received SPI frame 																					
				SPI_RcvData = SPI_RcvData  << 16;																						// shift first received SPI frame (higher) 								
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until second SPI frame is received 
				SPI_RcvData = SPI_RcvData + (LPC_SSP2->DR & 0x0000FFFF); 										// add lower received SPI frame
				
				os_mut_wait(&Mutuart_tx, 0xFFFF);																						// wait for mutex
				UART_SendFrame (0x01);																											// send identification frame to PC
				UART_SendFrame (SPI247_Buffer[SPI247_ReadBuff][0]);													// send target device
				UART_SendFrame (SPI_RcvData&0xFF);																					// send 1. byte (lowest)
				UART_SendFrame ((SPI_RcvData >> 8)&0xFF);																		// send 2. byte
				UART_SendFrame ((SPI_RcvData >> 16)&0xFF);																	// send 3. byte
				UART_SendFrame ((SPI_RcvData >> 24)&0xFF);																	// send 4. byte (highest)
				os_mut_release(&Mutuart_tx);																								// release mutex	
					
				SPI_SendFrame((SPI247_Buffer[SPI247_ReadBuff][0] & 0x07), 0, (SPI247_Buffer[SPI247_ReadBuff][1] & 0xF0) + 0x0F, 0, 0, 0);
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until first SPI frame is received 								
				SPI_RcvData = LPC_SSP2->DR;																									// received SPI frame 																					
				SPI_RcvData = SPI_RcvData  << 16;																						// shift first received SPI frame (higher) 								
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until second SPI frame is received 
				SPI_RcvData = SPI_RcvData + (LPC_SSP2->DR & 0x0000FFFF); 										// add lower received SPI frame
				
				if (((SPI247_Buffer[SPI247_ReadBuff][1]&0x0F) < 12))
				{
				if(((SPI_RcvData&0xFF) == SPI247_Buffer[SPI247_ReadBuff][2]) && (((SPI_RcvData>>8)&0xFF) == SPI247_Buffer[SPI247_ReadBuff][3]) && 
					(((SPI_RcvData>>16)&0xFF) == SPI247_Buffer[SPI247_ReadBuff][4]))
					temp++;
			  }	
					
				if((SPI247_Buffer[SPI247_ReadBuff][1] & 0x0F) == 0x0C)											// write to control register?
				{
						if ((SPI247_Buffer[SPI247_ReadBuff][2] & 0x05) == 0x05)									// odstraneno
						{
								while ((SPI_RcvData&0x05) == 5)																			// odstraneno
								{
										SPI_SendFrame((SPI247_Buffer[SPI247_ReadBuff][0] & 0x07), 0, SPI247_Buffer[SPI247_ReadBuff][1], SPI247_Buffer[SPI247_ReadBuff][2],
											SPI247_Buffer[SPI247_ReadBuff][3], SPI247_Buffer[SPI247_ReadBuff][4]); 		// send SPI frame
										while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until first SPI frame is received 								
										SPI_RcvData = LPC_SSP2->DR;																									// received SPI frame 																					
										SPI_RcvData = SPI_RcvData  << 16;																						// shift first received SPI frame (higher) 								
										while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until second SPI frame is received 
										SPI_RcvData = SPI_RcvData + (LPC_SSP2->DR & 0x0000FFFF); 										// add lower received SPI frame
								
								}
						}
				}
				os_mut_release(&Mutspi247);																									// release mutex
		}	// end of while(1)
}
//------------------------- TASK: spi247 -- (End) ----------------------------------


//------------------------- TASK: spi787 -- (Begin) --------------------------------
// SPI communication with source
__task void spi787 (void)
{   
		while(1)
		{
				os_sem_wait(&Semspi787, 0xFFFF);																											// wait for data (driven by semaphore)
				targetIC = SPI787_Buffer[SPI787_ReadBuff][0] & 0x03;																	// target device
				R_W = (SPI787_Buffer[SPI787_ReadBuff][0] & 0x08) >> 3;																// operation
				
				os_mut_wait(&Mutspi787, 0xFFFF);																											// wait for mutex
			   
      // odstraneno
      // odstraneno
							
			 SPI787_ReadBuff = (SPI787_ReadBuff + 1) & (BUFFERSPISIZE - 1);
							
				os_mut_release(&Mutspi787);																											// release mutex
			
		}
}
//------------------------- TASK: spi787 -- (End) ----------------------------------


//------------------------- TASK: dc247 -- (Begin) --------------------------------
// SPI communication with source
__task void dc247 (void)
{   
		while(1)
		{
				os_sem_wait(&Semdc247, 0xFFFF);																							// wait for data (driven by semaphore)
				for (i=0; i<12; i++)																												// delete data for dimming algorithm
				{
						DCs[i] = 0;
						ON[i] = 0;
						OFF[i] = 0;
						TR[i] = 15;
				}
					
				DCs[0] = 1023-(((DC247_Buffer[1]<<2) + (DC247_Buffer[2]>>6))&0x3FF);				// translate received data to duty-cycles
				DCs[1] = 1023-(((DC247_Buffer[2]<<4) + (DC247_Buffer[3]>>4))&0x3FF);
				DCs[2] = 1023-(((DC247_Buffer[3]<<6) + (DC247_Buffer[4]>>2))&0x3FF);							
				DCs[3] = 1023-(((DC247_Buffer[4]<<8) + (DC247_Buffer[5]))&0x3FF);
							
				DCs[4] = 1023-(((DC247_Buffer[6]<<2) + (DC247_Buffer[7]>>6))&0x3FF);
				DCs[5] = 1023-(((DC247_Buffer[7]<<4) + (DC247_Buffer[8]>>4))&0x3FF);
				DCs[6] = 1023-(((DC247_Buffer[8]<<6) + (DC247_Buffer[9]>>2))&0x3FF);							
				DCs[7] = 1023-(((DC247_Buffer[9]<<8) + (DC247_Buffer[10]))&0x3FF);
							
				DCs[8] = 1023-(((DC247_Buffer[11]<<2) + (DC247_Buffer[12]>>6))&0x3FF);
				DCs[9] = 1023-(((DC247_Buffer[12]<<4) + (DC247_Buffer[13]>>4))&0x3FF);
				DCs[10] = 1023-(((DC247_Buffer[13]<<6) + (DC247_Buffer[14]>>2))&0x3FF);							
				DCs[11] = 1023-(((DC247_Buffer[14]<<8) + (DC247_Buffer[15]))&0x3FF);	
					

				dimming(DCs, LEDcount, ON, OFF, TR, lastLevel[(DC247_Buffer[0]&0x07) - 3]);	// ON, OFF, TR values computation using Dimming algorithm
							
				os_mut_wait(&Mutspi247, 0xFFFF);																						// wait for mutex
				
				SPI_SendFrame((DC247_Buffer[0]&0x07), 1, 0xCF, 0, 0, 0);										// send SPI frame, read C register
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																						// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																								// sum received data	
									
				SPI_SendFrame((DC247_Buffer[0]&0x07), 1, 0xCC, ((SPI_RcvData)&(0xFB)), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);	// send SPI frame, read and write C
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																						// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																								// sum received data		
	 
									
				for(i=0; i<12; i++)																													// send ON, OFF and TR values via SPI
				{
						SPI_SendFrame((DC247_Buffer[0]&0x07), 1, i + 0xC0, ((OFF[i]<<4)+TR[i])&0xFF, ((ON[i]<<6)+(OFF[i]>>4))&0xFF, (ON[i]>>2)&0xFF);  
						while ((LPC_SSP2->SR & 0x4)== 0x0){}																		// wait until frame is received 
						SPI_RcvData = (LPC_SSP2->DR)<<16;																				// shift received data
						while ((LPC_SSP2->SR & 0x4)== 0x0){}																		// wait until frame is received 
						SPI_RcvData += LPC_SSP2->DR;																						// sum received data
				}
					
				SPI_SendFrame((DC247_Buffer[0]&0x07), 1, 0xCC, ((SPI_RcvData)&0xFF)|(1<<2), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);			//set  bit
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																						// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																								// sum received data	
							
				SPI_SendFrame((DC247_Buffer[0]&0x07), 1, 0xCF, 0, 0, 0);										// read C register
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																						// shift received data
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// wait until frame is received 
				SPI_RcvData += LPC_SSP2->DR;																								// sum received data		
									
				while(SPI_RcvData&(1<<2))																										// run until bit is cleared
				{
						SPI_SendFrame((DC247_Buffer[0]&0x07), 1, 0xCF, 0, 0, 0);
						while ((LPC_SSP2->SR & 0x4)== 0x0){}																		// wait until frame is received 
						SPI_RcvData = (LPC_SSP2->DR)<<16;																				// shift received data
						while ((LPC_SSP2->SR & 0x4)== 0x0){}																		// wait until frame is received 
						SPI_RcvData += LPC_SSP2->DR;																						// sum received data
				}
							
				os_mut_release(&Mutspi247);																									// release mutex			
				/*for(i=0; i<12; i++)
				{
						SPI_SendFrame((DC247_Buffer[0]&0x07), 1, i + 0xC0, ((OFF[i]<<4)+15) & 0xFF, ((ON[i]<<6)+(OFF[i]>>4)) & 0xFF, (ON[i]>>2) & 0xFF);  
						while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// cekej dokud neni prijmut SPI ramec 
						SPI_RcvData = (LPC_SSP2->DR)<<16;																									// prijaty ramec je ulozen do promenne SPI_RcvData
						while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// cekej dokud neni prijmut SPI ramec 
						SPI_RcvData += LPC_SSP2->DR;																									// prijaty ramec je ulozen do promenne SPI_RcvData 
				}
							
				SPI_SendFrame((DC247_Buffer[0]&0x07), 1, 0xCC, ((SPI_RcvData)&0xFF)|(1<<2), (SPI_RcvData>>8)&0xFF, (SPI_RcvData>>16)&0xFF);			//nastav bit
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// cekej dokud neni prijmut SPI ramec 
				SPI_RcvData = (LPC_SSP2->DR)<<16;																									// prijaty ramec je ulozen do promenne SPI_RcvData
				while ((LPC_SSP2->SR & 0x4)== 0x0){}																				// cekej dokud neni prijmut SPI ramec 
				SPI_RcvData += LPC_SSP2->DR;
				*/	
		
		}	// end of while(1)
}
//------------------------- TASK: dc247 -- (End) ----------------------------------


//------------------------- TASK: init -- (Begin) --------------------------------
// iniciation task, start other task
__task void init (void) 
{
	// create tasks:
			t_blink = os_tsk_create(blink, 0);
			t_uart_tx = os_tsk_create(uart_tx, 0); 
			t_interpreter = os_tsk_create(interpreter, 0);
			t_spi247 = os_tsk_create(spi247, 0);
			t_spi787 = os_tsk_create(spi787, 0);
			t_dc247 = os_tsk_create(dc247, 0);	

	// init semaphores:
			os_sem_init (&Semspi247, 0);
			os_sem_init (&Semspi787, 0);
			os_sem_init (&Semdc247, 0);
	
	// init mutexes
			os_mut_init(&Mutspi247);
			os_mut_init(&Mutspi787);
			os_mut_init(&Mutuart_tx);
	
	//delete itself:
			os_tsk_delete_self ();
}
//------------------------- TASK: init -- (End) ----------------------------------


//---------------- INTERRUPT: UART0_receive data -- (Begin) ----------------------
// receive data via UART from PC
void UART0_IRQHandler(void)	
	{	
		while((LPC_UART0->LSR) & 0x01)																	// run until data are ready
		{
		UART_RXBuffer[UART_RXWriteBuff] = LPC_UART0->RBR;								// receive data
		UART_RXWriteBuff = (UART_RXWriteBuff + 1) & (BUFFERSIZE-1);			// increment pointer
		UART_RXDataCount++;																							// increment data count
		}
	}
//----------------- INTERRUPT: UART0_receive data -- (End) -----------------------
	
	
//---------------------------- MAIN -- (Begin) -----------------------------------
// main function
	int main (void)
{
		GPIO_init();															// initialization of GPIO 
		SPI_init();																// initialization of SSP1 a SSP2 
		TIMER0_init();														// initialization of TIMER0 
		PWM_init();																// initialization of PWM1
		UART_init();															// initialization of UART0
		NVIC_EnableIRQ(UART0_IRQn);								// enable interrupt

		LED_off(4);																// turn off LED4
		LED_off(5);																// turn off LED5
		delay_ms(500);
		
		LED_on(4);																// turn on LED4
		LED_on(5);																// turn on LED5
		delay_ms(500);
	
		LED_off(4);																// turn off LED4
		LED_off(5);																// turn off LED5	
		delay_ms(500);
	
		LED_on(4);																// turn on LED4
		LED_on(5);																// turn on LED5
		delay_ms(500);
	
		LED_off(4);																// turn off LED4
		LED_off(5);																// turn off LED5

		os_sys_init(init);                        // Initialize RTX and start init
	
}
//----------------------------- MAIN -- (End) ------------------------------------